@@ -20,6 +20,8 @@ import json
 import logging
 import logging.config
 import os
+import os.path
+from pymongo import MongoClient
 import pwd
 import requests
 import re
@@ -31,7 +33,7 @@ import yaml
 from requests.exceptions import ConnectionError
 from ferry.table.prettytable import *
 from ferry.options import CmdHelp
-from ferry.install import Installer, FERRY_HOME, GUEST_DOCKER_REPO
+from ferry.install import Installer, FERRY_HOME, GUEST_DOCKER_REPO, DEFAULT_FERRY_APPS
 
 class CLI(object):
     def __init__(self):
@@ -45,7 +47,8 @@ class CLI(object):
         self.cmds.add_option("-k", "--key", "Specify key directory")
         self.cmds.add_option("-m", "--mode", "Deployment mode")
         self.cmds.add_option("-u", "--upgrade", "Upgrade Ferry")
-        self.cmds.add_option("-b", "--build", "Build Ferry images")
+        self.cmds.add_option("-b", "--build", "Build Ferry default images")
+        self.cmds.add_cmd("build", "Build a Dockerfile")
         self.cmds.add_cmd("clean", "Clean zombie Ferry processes")
         self.cmds.add_cmd("server", "Start all the servers")
         self.cmds.add_cmd("deploy", "Deploy a service to the cloud")
@@ -53,8 +56,12 @@ class CLI(object):
         self.cmds.add_cmd("info", "Print version information")
         self.cmds.add_cmd("inspect", "Return low-level information on a service")
         self.cmds.add_cmd("install", "Install the Ferry images")
+        self.cmds.add_cmd("login", "Login to Ferry servers")
         self.cmds.add_cmd("logs", "Copy over the logs to the host")
+        self.cmds.add_cmd("ls", "View installed applications")
         self.cmds.add_cmd("ps", "List deployed and running services")
+        self.cmds.add_cmd("pull", "Pull a remote image")
+        self.cmds.add_cmd("push", "Push an image to a remote registry")
         self.cmds.add_cmd("rm", "Remove a service or snapshot")
         self.cmds.add_cmd("snapshot", "Take a snapshot")
         self.cmds.add_cmd("snapshots", "List all snapshots")
@@ -67,10 +74,225 @@ class CLI(object):
         self.default_user = 'root'
         self.installer = Installer()
 
-    """
-    Create a new stack. 
-    """
+    def _pull_image(self, image):
+        """
+        Pull a remote image to the local registry. 
+        """
+        try:
+            payload = { 'image' : image } 
+            res = requests.get(self.ferry_server + '/image', params=payload)
+            return str(res.text)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
+
+    def _pull_app(self, app):
+        """
+        Pull a local application to Ferry servers. 
+        """
+        # Now download the application description
+        # from the Ferry servers. 
+        account, key, server = self.installer.get_ferry_account()
+        if account:
+            # Read in the contents of the application and
+            # generate the API key. 
+            req = { 'action' : 'fetch',
+                    'app' : app,
+                    'account' : account }
+            sig = self.installer.create_signature(json.dumps(req), key)
+        try:
+            payload = { 'id' : account,
+                        'app' : app, 
+                        'sig' : sig }
+            res = requests.get(server + '/app', params=payload)
+            status = json.loads(res.text)
+            file_name = self.installer.store_app(app, status['ext'], status['content'])
+            if file_name:
+                content = self._read_app_content(file_name)
+                images = self._get_user_images(content)
+                for i in images:
+                    self._pull_image(i)
+                return app
+            else:
+                return "failed"
+        except ConnectionError:
+            logging.error("could not connect to application server")
+            return "failed"
+
+    def _pull(self, image):
+        """
+        Pull a remote application/image to the local registry. 
+        """
+        logging.warning("pulling "  + image)
+
+        # Figure out if we're pushing a Ferry application or 
+        # plain Docker image. 
+        s = image.split("://")
+        if len(s) > 1:
+            proto = s[0]
+            image = s[1]
+        else:
+            proto = "image"
+            image = s[0]
+
+        if proto == "image":
+            return self._pull_image(image, registry)
+        else:
+            return self._pull_app(image)
+
+    def _push_image(self, image, registry):
+        """
+        Push a local image to a remote registry. 
+        """
+        try:
+            payload = { 'image' : image,
+                        'server' : registry } 
+            res = requests.post(self.ferry_server + '/image', data=payload)
+            return str(res.text)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
+
+    def _builtin_image(self, image):
+        """
+        Indicates whether the image is a pre-built Ferry image. 
+        Right now we only verify the client images. 
+        """
+        return image in ["ferry/hadoop-client",
+                         "ferry/spark-client",
+                         "ferry/cassandra-client",
+                         "ferry/openmpi-client"]
+
+    def _get_user_images(self, content):
+        """
+        Get the user-defined images.
+        """
+        images = set()
+        for c in content['connectors']:
+            p = c['personality']
+            if not self._builtin_image(p):
+                images.add(p)
+        return images
+
+    def _read_app_content(self, file_path):
+        """
+        Read the content of the application. 
+        """
+        json_arg = None
+        with open(file_path, "r") as f:
+            n, e = os.path.splitext(file_path)
+            if e == '.json':
+                json_string = self._read_file_arg(file_path)
+                json_arg = json.loads(json_string)
+            elif e == '.yaml' or e == '.yml':
+                yaml_file = open(file_path, 'r')
+                json_arg = yaml.load(yaml_file)
+        return json_arg
+
+    def _push_app(self, app, registry):
+        """
+        Push a local application to Ferry servers. 
+        """
+        # First find all the images that need to
+        # be pushed to the Docker registry. 
+        content = self._read_app_content(app)
+        if content:
+            images = self._get_user_images(content)
+            for i in images:
+                self._push_image(i, registry)
+        
+        # Register the application in the Ferry database. 
+        account, key, server = self.installer.get_ferry_account()
+        if account:
+            # Read in the contents of the application and
+            # generate the API key. 
+            with open(app, "r") as f:
+                name = account + '/' + os.path.basename(app)
+                name, ext = os.path.splitext(name)
+                content = f.read()
+                req = { 'action' : 'register',
+                        'app' : name,
+                        'account' : account }
+                sig = self.installer.create_signature(json.dumps(req), key)
+
+                try:
+                    payload = { 'id' : account,
+                                'app' : name, 
+                                'ext' : ext, 
+                                'content' : content,
+                                'sig' : sig }
+                    res = requests.post(server + '/app', data=payload)
+                    status = json.loads(res.text)
+                    if status['status'] == 'fail':
+                        logging.error("failed to register app " + app)
+                        return "Failed to register app " + app
+                    else:
+                        return status['name']
+                except ConnectionError:
+                    logging.error("could not connect to application server")
+                    return "Could not register the application."
+                except ValueError as e:
+                    logging.error(str(e))
+                    return "Registration server sent back unknown reply"
+        else:
+            logging.error("could not read account information")
+            return "Could not read account information."
+
+    def _push(self, image, registry):
+        """
+        Push a local appliation/image to a remote registry. 
+        """
+        logging.warning("pushing "  + image)
+
+        # Figure out if we're pushing a Ferry application or 
+        # plain Docker image. 
+        s = image.split("://")
+        if len(s) > 1:
+            proto = s[0]
+            image = s[1]
+        else:
+            proto = "image"
+            image = s[0]
+
+        if proto == "image":
+            return self._push_image(image, registry)
+        else:
+            return self._push_app(image, registry)
+
+    def _build(self, dockerfile):
+        """
+        Build a new local image. 
+        """
+        logging.warning("building "  + dockerfile)
+        names = self.installer._get_image(dockerfile)
+        name = names.pop().split("/")
+        if len(name) == 1:
+            repo = GUEST_DOCKER_REPO
+            image = name[0]
+        else:
+            repo = name[0]
+            image = name[1]
+
+        build_dir = os.path.dirname(dockerfile)
+        self.installer._compile_image(image, repo, build_dir, build=True)
+        if len(names) > 0:
+            self.installer._tag_images(image, repo, names)
+
+    def _login(self):
+        """
+        Login to a remote registry
+        """
+        try:
+            res = requests.post(self.ferry_server + '/login')
+            return str(res.text)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
+
     def _create_stack(self, stack_description, args):
+        """
+        Create a new stack. 
+        """
         mode = self._parse_deploy_arg('mode', args, default='local')
         conf = self._parse_deploy_arg('conf', args, default='default')
         payload = { 'payload' : json.dumps(stack_description),
@@ -99,6 +321,23 @@ class CLI(object):
             except ValueError as e:
                 logging.error(reply)
 
+    def _format_apps_query(self, json_data):
+        authors = []
+        versions = []
+        descriptions = []
+        for app in json_data.keys():
+            authors.append(json_data[app]['author'])
+            versions.append(json_data[app]['version'])
+            descriptions.append(json_data[app]['description'])            
+
+        t = PrettyTable()
+        t.add_column("App", json_data.keys())
+        t.add_column("Author", authors)
+        t.add_column("Version", versions)
+        t.add_column("Description", descriptions)
+        return t.get_string(sortby="App",
+                            padding_width=2)
+        
     def _format_snapshots_query(self, json_data):
         bases = []
         date = []
@@ -190,7 +429,22 @@ class CLI(object):
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
+    def _list_apps(self):
+        """
+        List all installed applications including the built-in applications.
+        """
+        try:
+            res = requests.get(self.ferry_server + '/apps')
+            json_reply = json.loads(res.text)
+            return self._format_apps_query(json_reply)
+        except ConnectionError:
+            logging.error("could not connect to ferry server")
+            return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
+
     def _list_snapshots(self):
+        """
+        List all snapshots.
+        """
         try:
             res = requests.get(self.ferry_server + '/snapshots')
             json_reply = json.loads(res.text)
@@ -204,22 +458,23 @@ class CLI(object):
                           sort_keys=True,
                           indent=2,
                           separators=(',',':'))
-    """
-    Inspect a specific stack. 
-    """
+
     def _inspect_stack(self, stack_id):
+        """
+        Inspect a specific stack. 
+        """
         payload = { 'uuid':stack_id }
         try:
             res = requests.get(self.ferry_server + '/stack', params=payload)
-            json_value = json.loads(str(res.text))
-            return self._format_stack_inspect(json_value)
+            return res.text
         except ConnectionError:
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
-    """
-    Copy over the logs. 
-    """
+
     def _copy_logs(self, stack_id, to_dir):
+        """
+        Copy over the logs. 
+        """
         payload = {'uuid':stack_id,
                    'dir':to_dir}
         try:
@@ -273,10 +528,10 @@ class CLI(object):
                 return m.group(1)
         return default
 
-    """
-    Deploy the stack. 
-    """        
     def _deploy_stack(self, stack_id, args):
+        """
+        Deploy the stack. 
+        """
         mode = self._parse_deploy_arg('mode', args, default='local')
         conf = self._parse_deploy_arg('conf', args, default='default')
 
@@ -290,10 +545,10 @@ class CLI(object):
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
 
-    """
-    Manage the stack. 
-    """
     def _manage_stacks(self, stack_info):
+        """
+        Manage the stack. 
+        """
         try:
             res = requests.post(self.ferry_server + '/manage/stack', data=stack_info)
             return str(res.text)        
@@ -301,16 +556,16 @@ class CLI(object):
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
         
-    """
-    Output the help message.
-    """
     def _print_help(self):
+        """
+        Output the help message.
+        """
         return self.cmds.print_help()
 
-    """
-    Output version information.
-    """
     def _print_info(self):
+        """
+        Output version information.
+        """
         try:
             res = requests.get(self.ferry_server + '/version')
             s = self.cmds.description + '\n'
@@ -322,10 +577,10 @@ class CLI(object):
             logging.error("could not connect to ferry server")
             return "It appears Ferry servers are not running.\nType sudo ferry server and try again."
         
-    """
-    Helper method to read a file.
-    """
     def _read_file_arg(self, file_name):
+        """
+        Helper method to read a file.
+        """
         json_file = open(os.path.abspath(file_name), 'r')
         json_text = ''
 
@@ -334,11 +589,11 @@ class CLI(object):
 
         return json_text
 
-    """
-    Read the location of the directory containing the keys
-    used to communicate with the containers. 
-    """
     def _read_key_dir(self):
+        """
+        Read the location of the directory containing the keys
+        used to communicate with the containers. 
+        """
         f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
         k = f.read().strip().split("://")
         return k[1], k[0]
@@ -380,62 +635,49 @@ class CLI(object):
                                       ferry.install.GLOBAL_KEY_DIR)
             logging.warning("Copied ssh keys to " + ferry.install.GLOBAL_KEY_DIR)
 
-    """
-    This is the command dispatch table. 
-    """
+
+    def _find_installed_app(self, app):
+        """
+        Help find the path to the application. Check both the built-in
+        global directory and the user-installed directory.
+        """
+        file_path = None
+        for item in os.listdir(FERRY_HOME + '/data/plans/'):
+            if app == os.path.splitext(item)[0]:
+                return FERRY_HOME + '/data/plans/' + item
+
+        if not file_path:
+            for user in os.listdir(DEFAULT_FERRY_APPS):
+                if os.path.isdir(DEFAULT_FERRY_APPS + '/' + user):
+                    for item in os.listdir(DEFAULT_FERRY_APPS + '/' + user):
+                        if app == user + '/' + os.path.splitext(item)[0]:
+                            return DEFAULT_FERRY_APPS + '/' + user + '/' + item
+
     def dispatch_cmd(self, cmd, args, options):
+        """
+        This is the command dispatch table. 
+        """
         if(cmd == 'start'):
             self._check_ssh_key()
 
+            # Check if we need to build the image before running. 
             if '-b' in options:
                 build_dir = options['-b'][0]
-                dockerfile = build_dir + '/Dockerfile'
-                names = self.installer._get_image(dockerfile)
-                name = names.pop().split("/")
-                if len(name) == 1:
-                    repo = GUEST_DOCKER_REPO
-                    image = name[0]
-                else:
-                    repo = name[0]
-                    image = name[1]
-                    
-                self.installer._compile_image(image, repo, build_dir, build=True)
-                if len(names) > 0:
-                    self.installer._tag_images(image, repo, names)
+                self._build(build_dir + '/Dockerfile')
 
             arg = args.pop(0)
             json_arg = {}
             if not os.path.exists(arg):
-                # Check if the user wants to use one of the global plans.
-                global_path = FERRY_HOME + '/data/plans/' + arg
-
-                # If the user has not supplied a file extension, look for the
-                # file with a YAML extension
-                file_path = global_path
-                n, e = os.path.splitext(global_path)
-                if e == '':
-                    file_path += '.yaml'
+                file_path = self._find_installed_app(arg)
             else:
                 file_path = arg
 
-            if os.path.exists(file_path):
+            if file_path and os.path.exists(file_path):
                 file_path = os.path.abspath(file_path)
-                n, e = os.path.splitext(file_path)
-                if e == '.json':
-                    json_string = self._read_file_arg(file_path)
-                    json_arg = json.loads(json_string)
-                elif e == '.yaml' or e == '.yml':
-                    yaml_file = open(file_path, 'r')
-                    json_arg = yaml.load(yaml_file)
-                else:
-                    # This is an unknown file type. We're going to try to
-                    # pretend it is a JSON file and see if we fail.
-                    try:
-                        json_string = self._read_file_arg(file_path)
-                        json_arg = json.loads(json_string)
-                    except ValueError as e:
-                        logging.error("could not load file " + file_path)
-                        exit(1)
+                json_arg = self._read_app_content(file_path)
+                if not json_arg:
+                    logging.error("could not load file " + file_path)
+                    exit(1)
 
                 json_arg['_file_path'] = file_path
             json_arg['_file'] = arg
@@ -499,8 +741,23 @@ class CLI(object):
             self._check_ssh_key()
             stack_id = args.pop(0)
             return self._deploy_stack(stack_id, args)
+        elif(cmd == 'ls'):
+            return self._list_apps()
         elif(cmd == 'info'):
             return self._print_info()
+        elif(cmd == 'build'):
+            return self._build(args[0])
+        elif(cmd == 'pull'):
+            return self._pull(args[0])
+        elif(cmd == 'push'):
+            image = args.pop(0)
+            if len(args) > 0:
+                registry = args.pop(0)
+            else:
+                registry = None
+            return self._push(image, registry)
+        elif(cmd == 'login'):
+            return self._login()
         elif(cmd == 'help'):
             return self._print_help()
         else:
