@@ -76,6 +76,7 @@ class CassandraInitializer(object):
         ports.append(CassandraConfig.THRIFT_COM_PORT)
         ports.append(CassandraConfig.NATIVE_COM_PORT)
         ports.append(CassandraConfig.JMX_PORT)
+        ports.append(CassandraConfig.RMI_PORT)
 
         return ports
 
@@ -200,13 +201,14 @@ class CassandraConfig(object):
     log_directory = '/service/logs'
     commit_directory = '/service/data/commits'
     cache_directory = '/service/data/cache'
-    config_directory = '/service/conf'
+    config_directory = '/service/conf/cassandra'
 
     CLUSTER_COM_PORT = 7000
     CLUSTER_SSL_PORT = 7001
     THRIFT_COM_PORT  = 9160
     NATIVE_COM_PORT  = 9042
     JMX_PORT         = 7199
+    RMI_PORT         = 7200
 
     def __init__(self, num):
         self.num = num
