@@ -193,7 +193,7 @@ A GlusterFS configuration.
 class GlusterConfig(object):
     data_directory = '/service/data'
     log_directory = '/service/logs'
-    config_directory = '/service/configuration'
+    config_directory = '/service/conf/gluster'
 
     BRICK_PORT = 24009
     MANAGEMENT_PORT = 24007
