@@ -13,10 +13,11 @@
 # limitations under the License.
 #
 
-import os
 import json
 import logging
+import os
 import re
+import sys
 from subprocess import Popen, PIPE
 
 DOCKER_SOCK='unix:////var/run/ferry.sock'
@@ -83,11 +84,13 @@ class DockerCLI(object):
         self.images_cmd = 'images'
         self.commit_cmd = 'commit'
         self.push_cmd = 'push'
+        self.pull_cmd = 'pull'
         self.stop_cmd = 'stop'
         self.tag_cmd = 'tag'
         self.rm_cmd = 'rm'
         self.ps_cmd = 'ps'
         self.info_cmd = 'info'
+        self.login_cmd = 'login'
         self.daemon = '-d'
         self.interactive = '-i'
         self.tty = '-t'
@@ -167,32 +170,78 @@ class DockerCLI(object):
         cmd = data[0]['Config']['Cmd']
         return json.dumps( {'Cmd' : cmd} )
 
-    """
-    Push an image to a remote registry.
-    """
-    def push(self, container, registry):
-        raw_image_name = container.image.split("/")[1]
-        new_image = "%s/%s" % (registry, raw_image_name)
+    def login(self, user, password, email, registry):
+        """
+        Login to a remote registry. 
+        """
+        cmd = self.docker + ' ' + self.login_cmd + ' -u %s -p %s -e %s %s' % (user, password, email, registry)
+        logging.warning(cmd)
+        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        if output.strip() == "Login Succeeded":
+            return True
+        else:
+            logging.error(output.strip())
+            return False
 
-        tag = self.docker + ' ' + self.tag_cmd + ' ' + container.image + ' ' + new_image
+    def _continuous_print(self, process, msg):
+        while True:
+            try:
+                out = process.stdout.read(161)
+                if out == '':
+                    break
+                else:
+                    logging.warning(out)
+            except IOError as e:
+                logging.warning(e)
+
+        try:
+            errmsg = process.stderr.readline()
+            if errmsg and errmsg != '':
+                logging.error(errmsg)
+                return False
+            else:
+                logging.warning("downloaded image!")
+        except IOError:
+            pass
+        return True
+
+    def push(self, image, registry=None):
+        """
+        Push an image to a remote registry.
+        """
+        if registry:
+            raw_image_name = image.split("/")[1]
+            new_image = "%s/%s" % (registry, raw_image_name)
+            tag = self.docker + ' ' + self.tag_cmd + ' ' + image + ' ' + new_image
+            logging.warning(tag)
+            Popen(tag, shell=True)
+        else:
+            new_image = image
         push = self.docker + ' ' + self.push_cmd + ' ' + new_image
-        logging.warning(tag)
         logging.warning(push)
-        
-        Popen(tag, stdout=PIPE, shell=True).stdout.read()
-        Popen(push, stdout=PIPE, shell=True).stdout.read()
+        child = Popen(push, stdout=PIPE, stderr=PIPE, shell=True)
+        return self._continuous_print(child, "uploading image...")
+
+    def pull(self, image):
+        """
+        Pull a remote image to the local registry. 
+        """
+        pull = self.docker + ' ' + self.pull_cmd + ' ' + image
+        logging.warning(pull)
+        child = Popen(pull, stdout=PIPE, stderr=PIPE, shell=True)
+        return self._continuous_print(child, "downloading image...")
 
-    """
-    Commit a container
-    """
     def commit(self, container, snapshot_name):
+        """
+        Commit a container
+        """
         default_run = self._get_default_run(container)
         run_cmd = "-run='%s'" % default_run
 
         # Construct a new container using the given snapshot name. 
         cmd = self.docker + ' ' + self.commit_cmd + ' ' + run_cmd + ' ' + container.container + ' ' + snapshot_name
         logging.warning(cmd)
-        output = Popen(cmd, stdout=PIPE, shell=True).stdout.read()
+        Popen(cmd, shell=True)
 
     """
     Stop a running container
