@@ -25,6 +25,7 @@ import stat
 import sys
 import time
 import uuid
+import yaml
 from pymongo import MongoClient
 from sets import Set
 from ferry.install import *
@@ -217,10 +218,41 @@ class DockerManager(object):
             for i in c['instances']:
                 self._copy_instance_logs(i, storage_dir)
 
-    """
-    Inspect a deployed stack. 
-    """
+    def _inspect_application(self, app, directory):
+        """
+        Read out the contents of an application
+        """
+        s = app.split('/')
+        if len(s) > 1:
+            name = s[1]
+        else:
+            name = s[0]
+
+        dir_path = os.path.dirname(os.path.join(directory, app))
+        if os.path.exists(dir_path):
+            for item in os.listdir(dir_path):
+                n, e = item.split('.')
+                if n == name:
+                    with open(os.path.join(dir_path, item), 'r') as f:
+                        return f.read()
+
+    def inspect_installed(self, app):
+        """
+        Inspect an installed application
+        """
+        content = self._inspect_application(app, ferry.install.DEFAULT_BUILTIN_APPS)
+        if not content:
+            content = self._inspect_application(app, ferry.install.DEFAULT_FERRY_APPS)
+        
+        if content:
+            return content
+        else:
+            return ""
+        
     def inspect_deployed(self, uuid, registry):
+        """
+        Inspect a deployed stack. 
+        """
         json_reply = {}
 
         # Need to inspect the registry to make sure
@@ -231,10 +263,10 @@ class DockerManager(object):
                           indent=2,
                           separators=(',',':'))
 
-    """
-    Inspect a running stack. 
-    """
     def inspect_stack(self, stack_uuid):
+        """
+        Inspect a running stack. 
+        """
         json_reply = {}
 
         # Get the collection of all backends and connector UUIDS.
@@ -270,16 +302,73 @@ class DockerManager(object):
 
         # Now append some snapshot info. 
         json_reply['snapshots'] = self._get_snapshot_info(stack_uuid)
-
+           
+        json_reply['status'] = 'stack'
         return json.dumps(json_reply, 
                           sort_keys=True,
                           indent=2,
                           separators=(',',':'))
 
-    """
-    Query the available snapshots. 
-    """
+    def _read_file_arg(self, file_name):
+        """
+        Helper method to read a file.
+        """
+        json_file = open(os.path.abspath(file_name), 'r')
+        json_text = ''
+
+        for line in json_file:
+            json_text += line.strip()
+
+        return json_text
+
+    def _list_applications(self, directory):
+        apps = set()
+        content = os.listdir(directory)
+        for c in content:
+            if os.path.isdir(os.path.join(directory, c)):
+                user_apps = self._list_applications(os.path.join(directory, c))
+                for u in user_apps:
+                    apps.add(c + '/' + u)
+            else:
+                apps.add(c)
+        return apps
+
+    def _query_application(self, directory):
+        apps = {}
+        app_files = self._list_applications(directory)
+        for f in app_files:
+            file_path = directory + '/' + f
+            n, e = f.split(".")
+            content = None
+            if e == 'json':
+                json_string = self._read_file_arg(file_path)
+                content = json.loads(json_string)
+            elif e == 'yaml' or e == 'yml':
+                yaml_file = open(file_path, 'r')
+                content = yaml.load(yaml_file)
+
+            if content and 'metadata' in content:
+                apps[n] = { 'author' : content['metadata']['author'],
+                            'version': content['metadata']['version'],
+                            'description': content['metadata']['description'][:21] + "..." }
+            else:
+                apps[n] = { 'author' : "Unknown",
+                            'version': "Unknown",
+                            'description': "Unknown" }
+        return apps
+    
+    def query_applications(self):
+        """
+        Get list of installed applications.
+        """
+        builtin = self._query_application(ferry.install.DEFAULT_BUILTIN_APPS)
+        installed = self._query_application(ferry.install.DEFAULT_FERRY_APPS)
+        return json.dumps(dict(builtin.items() + installed.items()))
+        
     def query_snapshots(self, constraints=None):
+        """
+        Query the available snapshots. 
+        """
         json_reply = {}
 
         values = self.snapshot_collection.find()
@@ -382,10 +471,10 @@ class DockerManager(object):
         else:
             return False
 
-    """
-    Check if the application status
-    """
     def _confirm_status(self, uuid, status):
+        """
+        Check if the application status
+        """
         cluster = self.cluster_collection.find_one( {'uuid':uuid} )
         if cluster:
             return cluster['status'] == status
@@ -396,11 +485,11 @@ class DockerManager(object):
         return self._confirm_status(uuid, 'stopped')
     def is_removed(self, uuid, conf=None):
         return self._confirm_status(uuid, 'removed')
-
-    """
-    Check if the UUID is of a deployed application. 
-    """
+    
     def is_deployed(self, uuid, conf=None):
+        """
+        Check if the UUID is of a deployed application. 
+        """
         v = self.deploy.find( one=True, 
                               spec = {'uuid':uuid},
                               conf = conf )
@@ -409,19 +498,26 @@ class DockerManager(object):
         else:
             return False
 
-    """
-    Get the base image of this cluster. 
-    """
+    def is_installed(self, app):
+        """
+        Indicate whether the application is installed. 
+        """
+        apps = self.query_applications()
+        return app in apps
+
     def get_base_image(self, uuid):
+        """
+        Get the base image of this cluster. 
+        """
         cluster = self.cluster_collection.find_one( {'uuid':uuid} )
         if cluster:
             return cluster['base']
         return None
 
-    """
-    Create a new data directory
-    """
     def _new_data_dir(self, service_uuid, storage_type, storage_id):
+        """
+        Create a new data directory
+        """
         # Check the location of the scratch directory. If not defined
         # use the current directory
         if 'FERRY_SCRATCH' in os.environ:
@@ -692,10 +788,10 @@ class DockerManager(object):
                 for c in s['containers']:
                     self.docker.stop([c])
 
-    """
-    Register the set of services under a single cluster identifier. 
-    """
     def register_stack(self, backends, connectors, base, uuid=None):
+        """
+        Register the set of services under a single cluster identifier. 
+        """
         if not uuid:
             cluster_uuid = self._new_stack_uuid()
         else:
@@ -719,10 +815,10 @@ class DockerManager(object):
 
         return cluster_uuid
 
-    """
-    Helper method to update a cluster's status. 
-    """
     def _update_stack(self, cluster_uuid, state):
+        """
+        Helper method to update a cluster's status. 
+        """
         self.cluster_collection.update( {'uuid' : cluster_uuid},
                                         {'$set' : state} )
 
@@ -772,10 +868,10 @@ class DockerManager(object):
                             all_compute.append(compute)
             return all_connectors, all_compute, all_storage
 
-    """
-    Stop a running cluster.
-    """
     def _stop_stack(self, cluster_uuid):
+        """
+        Stop a running cluster.
+        """
         # First stop all the running services. 
         connectors, compute, storage = self._get_cluster_instances(cluster_uuid)
         for c in connectors:
@@ -810,10 +906,10 @@ class DockerManager(object):
         for v in volumes:
             shutil.rmtree(v)
     
-    """
-    Take a snapshot of an existing stack. 
-    """
     def _snapshot_stack(self, cluster_uuid):
+        """
+        Take a snapshot of an existing stack. 
+        """
         cluster = self.cluster_collection.find_one( {'uuid':cluster_uuid} )
         if cluster:
             # We need to deserialize the docker containers from the cluster/service
@@ -851,15 +947,15 @@ class DockerManager(object):
         service_info = self._get_service_configuration(uuid, detailed=True)
         self._start_service(uuid, containers, service_info)
 
-    """
-    Allocate a new compute cluster.
-    """
     def allocate_compute(self,
                          compute_type, 
                          storage_uuid,
                          args, 
                          num_instances=1,
                          layers=[]):
+        """
+        Allocate a new compute cluster.
+        """
         # Allocate a UUID.
         service_uuid = self._new_service_uuid()
         service = self._get_service(compute_type)
@@ -960,16 +1056,16 @@ class DockerManager(object):
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
                         
-    """
-    Create a storage cluster and start a particular
-    personality on that cluster. 
-    """
     def allocate_storage(self, 
                          storage_type, 
                          num_instances=1,
                          layers=[], 
                          args=None,
                          replace=False):
+        """
+        Create a storage cluster and start a particular
+        personality on that cluster. 
+        """
         # Allocate a UUID.
         service_uuid = self._new_service_uuid()
         service = self._get_service(storage_type)
@@ -1000,16 +1096,16 @@ class DockerManager(object):
         self._update_service_configuration(service_uuid, service)
         return service_uuid, containers
 
-    """
-    Get the default deployment conf file. 
-    """
     def _get_default_conf(self):        
+        """
+        Get the default deployment conf file. 
+        """
         return FERRY_HOME + '/data/conf/deploy_default.json'
 
-    """
-    Get the deployment configuration parameters. 
-    """
     def _get_deploy_params(self, mode, conf):
+        """
+        Get the deployment configuration parameters. 
+        """
         # First just find and read the configuration file. 
         if conf == 'default':
             conf = self._get_default_conf()
@@ -1025,10 +1121,10 @@ class DockerManager(object):
                     return j[mode]
         return None
 
-    """
-    Deploy an existing stack. 
-    """
     def deploy_stack(self, cluster_uuid, params=None):
+        """
+        Deploy an existing stack. 
+        """
         containers = []
         cluster = self.cluster_collection.find_one( {'uuid':cluster_uuid} )
         if cluster:
@@ -1045,12 +1141,13 @@ class DockerManager(object):
             if params and 'start-on-create' in params:
                 return True
         return False
-    """
-    Manage the stack.
-    """
+
     def manage_stack(self,
                      stack_uuid,
                      action):
+        """
+        Manage the stack.
+        """
         status = 'running'
         if(action == 'snapshot'):        
             # The user wants to take a snapshot of the current stack. This
@@ -1078,10 +1175,10 @@ class DockerManager(object):
                  'status' : True,
                  'msg': status }
 
-    """
-    Lookup the stopped backend info. 
-    """
     def fetch_stopped_backend(self, uuid):
+        """
+        Lookup the stopped backend info. 
+        """
         cluster = self.cluster_collection.find_one( {'uuid':uuid} )
         if cluster:
             backends = []
@@ -1101,19 +1198,19 @@ class DockerManager(object):
             return backends
             
 
-    """
-    Lookup the snapshot backend info. 
-    """
     def fetch_snapshot_backend(self, snapshot_uuid):
+        """
+        Lookup the snapshot backend info. 
+        """
         snapshot = self.cluster_collection.find_one( {'snapshot_uuid':snapshot_uuid} )
         if snapshot:
             logging.warning("SNAPSHOT BACKEND: " + str(snapshot['backends']))
             return snapshot['backends']['backend']
 
-    """
-    Lookup the deployed backend info. 
-    """
     def fetch_deployed_backend(self, app_uuid, conf=None):
+        """
+        Lookup the deployed backend info. 
+        """
         app = self.deploy.find( one = True,
                                 spec = { 'uuid' : app_uuid },
                                 conf = conf )
@@ -1121,13 +1218,13 @@ class DockerManager(object):
         if stack:
             return stack['backends']['backend']
 
-    """
-    Lookup the deployed application connector info and instantiate. 
-    """
     def allocate_stopped_connectors(self, 
                                      app_uuid, 
                                      backend_info,
                                      conf = None):
+        """
+        Lookup the deployed application connector info and instantiate. 
+        """
         connector_info = []
         connector_plan = []
         cluster = self.cluster_collection.find_one( {'uuid':app_uuid} )
@@ -1147,13 +1244,13 @@ class DockerManager(object):
                     self.restart_containers(cuid, containers)
         return connector_info, connector_plan
 
-    """
-    Lookup the deployed application connector info and instantiate. 
-    """
     def allocate_deployed_connectors(self, 
                                      app_uuid, 
                                      backend_info,
                                      conf = None):
+        """
+        Lookup the deployed application connector info and instantiate. 
+        """
         connector_info = []
         connector_plan = []
         app = self.deploy.find( one = True,
@@ -1174,12 +1271,12 @@ class DockerManager(object):
                                          'start' : 'start' } )
         return connector_info, connector_plan
                 
-    """
-    Lookup the snapshot connector info and instantiate. 
-    """
     def allocate_snapshot_connectors(self, 
                                      snapshot_uuid, 
                                      backend_info):
+        """
+        Lookup the snapshot connector info and instantiate. 
+        """
         connector_info = []
         connector_plan = []
         snapshot = self.snapshot_collection.find_one( {'snapshot_uuid':snapshot_uuid} )
@@ -1199,13 +1296,13 @@ class DockerManager(object):
         return connector_info, connector_plan
                 
 
-    """
-    Restart a stopped connector with an existing storage service. 
-    """
     def _restart_connectors(self,
                             service_uuid, 
                             connectors, 
                             backend=None):
+        """
+        Restart a stopped connector with an existing storage service. 
+        """
         # Initialize the connector and connect to the storage. 
         storage_entry = []
         compute_entry = []
@@ -1246,9 +1343,6 @@ class DockerManager(object):
                    'status':'running'}
         self._update_service_configuration(service_uuid, service)
                 
-    """
-    Allocate a new connector and associate with an existing storage service. 
-    """
     def allocate_connector(self,
                            connector_type, 
                            backend=None,
@@ -1256,6 +1350,9 @@ class DockerManager(object):
                            args=None,
                            ports=None, 
                            image=None):
+        """
+        Allocate a new connector and associate with an existing storage service. 
+        """
         # Initialize the connector and connect to the storage. 
         storage_entry = []
         compute_entry = []
@@ -1315,8 +1412,26 @@ class DockerManager(object):
         self._update_service_configuration(service_uuid, service_info)
         return service_uuid, containers
 
-    """
-    Fetch the current docker version.
-    """
+    def push_image(self, image, registry=None):
+        """
+        Push a local image to a remote registry.         
+        """
+        return self.docker.push(image, registry)
+
+    def pull_image(self, image):
+        """
+        Pull a remote image to the local registry. 
+        """
+        return self.docker.pull(image)
+
+    def login_registry(self):
+        """
+        Login to a remote registry.
+        """
+        return self.docker.login()
+
     def version(self):
+        """
+        Fetch the current docker version.
+        """
         return self.docker.version()
