@@ -376,7 +376,6 @@ def _register_ip_addresses(backend_plan, connector_plan):
             # when restarting we get dictionary descriptions. Should just
             # fix at the restart level! 
             if isinstance(c, dict):
-                logging.warning("CONTAINER: " + str(c))
                 ips.append( [c['internal_ip'], c['hostname']] )
             else:
                 ips.append( [c.internal_ip, c.host_name] )
@@ -420,7 +419,6 @@ def _allocate_new(payload):
     reply = {}
     backend_info, backend_plan = _allocate_backend(payload, replace=True)
     reply['status'] = backend_info['status']
-    logging.warning("NEW BACKEND: " + str(backend_info))
     if backend_info['status'] == 'ok':
         success, connector_info, connector_plan = _allocate_connectors(payload, backend_info['uuids'])
 
@@ -458,7 +456,6 @@ def _allocate_snapshot(payload):
     """
     Helper function to allocate and start a snapshot.
     """
-    logging.warning("ALLOCATE SNAPSHOT " + str(payload['_file']))
     backend_info, backend_plan = _allocate_backend_from_snapshot(payload)
     if backend_info['status'] == 'ok':
         connector_info, connector_plan = _allocate_connectors_from_snapshot(payload, 
@@ -484,11 +481,48 @@ def _allocate_deployed(payload, params=None):
         return json.dumps({'status' : 'ok',
                            'text' : str(uuid)})
 
-"""
-Create some new storage infrastructure
-"""
+
+@app.route('/login', methods=['POST'])
+def login_registry():
+    """
+    Login to a remote registry. 
+    """
+    if docker.login_registry():
+        return "success"
+    else:
+        return "fail"
+
+@app.route('/image', methods=['POST'])
+def push_image():
+    """
+    Push a local image to a remote registry. 
+    """
+    image = request.form['image']
+    if 'server' in request.form:
+        registry = request.form['server']
+    else:
+        registry = None
+    if docker.push_image(image, registry):
+        return "success"
+    else:
+        return "fail"
+
+@app.route('/image', methods=['GET'])
+def pull_image():
+    """
+    Pull a remote image to the local registry. 
+    """
+    image = request.args['image']
+    if docker.pull_image(image):
+        return "success"
+    else:
+        return "fail"
+    
 @app.route('/create', methods=['POST'])
 def allocate_stack():
+    """
+    Create some new storage infrastructure
+    """
     payload = json.loads(request.form['payload'])
     mode = request.form['mode']
     conf = request.form['conf']
@@ -507,41 +541,52 @@ def allocate_stack():
     else:
         return "Could not start " + payload['_file']
 
-"""
-Query the deployed applications.
-"""
 @app.route('/deployed', methods=['GET'])
 def query_deployed():
+    """
+    Query the deployed applications.
+    """
     mode = request.args['mode']
     conf = request.args['conf']
     params = docker._get_deploy_params(mode, conf)
 
     return docker.query_deployed(params)
 
-"""
-Query the stacks.
-"""
 @app.route('/query', methods=['GET'])
 def query_stacks():
+    """
+    Query the stacks.
+    """
     if 'constraints' in request.args:
         constraints = json.loads(request.args['constraints'])
         return docker.query_stacks(constraints)
     else:
         return docker.query_stacks()
-"""
-Query the snapshots
-"""
+
 @app.route('/snapshots', methods=['GET'])
 def snapshots():
+    """
+    Query the snapshots
+    """
     return docker.query_snapshots()
 
-"""
-Inspect a particular stack.
-"""
+@app.route('/apps', methods=['GET'])
+def apps():
+    """
+    Get list of installed applications.
+    """
+    return docker.query_applications()
+
 @app.route('/stack', methods=['GET'])
 def inspect():
+    """
+    Inspect a particular stack.
+    """
     uuid = request.args['uuid']
-    return docker.inspect_stack(uuid)
+    if docker.is_running(uuid) or docker.is_stopped(uuid):
+        return docker.inspect_stack(uuid)
+    elif docker.is_installed(uuid):
+        return docker.inspect_installed(uuid)
 
 """
 Copy over logs
