@@ -13,19 +13,24 @@
 # limitations under the License.
 #
 
+import base64
 import ferry
 import grp
+import hashlib
+import hmac
 import json
 import logging
 import os
 import os.path
 import pwd
 import re
+import sh
 import shutil
 import stat
 import struct
 import sys
 import time
+import yaml
 from distutils import spawn
 from string import Template
 from subprocess import Popen, PIPE
@@ -55,7 +60,27 @@ def _supported_arch():
 
 def _supported_lxc():
     output = Popen("(lxc-version 2>/dev/null || lxc-start --version) | sed 's/.* //'", stdout=PIPE, shell=True).stdout.read()
-    ver = tuple(map(int, (output.strip().split(".")[:3])))
+
+    # Ignore all non-numeric strings in the
+    # versioning information. 
+    cleaned = []
+    tuples = output.strip().split(".")[:3]
+    for t in tuples: 
+        m = re.compile('(\d)*').match(t)
+        if m and len(m.groups()) > 0 and m.group(1) != '':
+            cleaned.append(m.group(1))
+        else:
+            cleaned.append(str(0))
+
+    # We need our tuples to be exactly three values. If
+    # there are more values, add a zero. 
+    for i in range(3 - len(cleaned)):
+        cleaned.append(0)
+
+    # Now compare the tuples. We need at least version 0.7.5. This
+    # assumes that lxc-version info is consistent across distributions
+    # which may not be true...
+    ver = tuple(map(int, cleaned))
     return ver > (0, 7, 5)
 
 def _supported_python():
@@ -78,6 +103,7 @@ FERRY_HOME=_get_ferry_home()
 DEFAULT_IMAGE_DIR=FERRY_HOME + '/data/dockerfiles'
 DEFAULT_KEY_DIR=FERRY_HOME + '/data/key'
 GLOBAL_KEY_DIR=DEFAULT_KEY_DIR
+DEFAULT_DOCKER_LOGIN=os.environ['HOME'] + '/.ferry.docker'
 DEFAULT_DOCKER_REPO='ferry'
 GUEST_DOCKER_REPO='ferry-user'
 DEFAULT_FERRY_OWNER='ferry:docker'
@@ -85,6 +111,8 @@ DOCKER_CMD='docker-ferry'
 DOCKER_SOCK='unix:////var/run/ferry.sock'
 DOCKER_DIR='/var/lib/ferry'
 DOCKER_PID='/var/run/ferry.pid'
+DEFAULT_BUILTIN_APPS=FERRY_HOME + '/data/plans'
+DEFAULT_FERRY_APPS='/var/lib/ferry/apps'
 DEFAULT_MONGO_DB='/var/lib/ferry/mongo'
 DEFAULT_MONGO_LOG='/var/lib/ferry/mongolog'
 DEFAULT_REGISTRY_DB='/var/lib/ferry/registry'
@@ -95,7 +123,45 @@ class Installer(object):
     def __init__(self):
         self.network = DHCPClient()
 
+    def get_ferry_account(self):
+        """
+        Read in the remote Ferry DB account information. Used
+        for registering applications. 
+        """
+        with open(ferry.install.DEFAULT_DOCKER_LOGIN, 'r') as f:
+            args = yaml.load(f)
+            args = args['ferry']
+            if all(k in args for k in ("user","key","server")):
+                return args['user'], args['key'], args['server']
+        return None, None, None
+
+    def create_signature(self, request, key):
+        """
+        Generated a signed request.
+        """
+        return hmac.new(key, request, hashlib.sha256).hexdigest()
+
+    def store_app(self, app, ext, content):
+        """
+        Store the application in the global directory. 
+        """
+        try:
+            # We may need to create the parent directory
+            # if this is the first time an application from this user
+            # is downloaded. 
+            file_name = os.path.join(DEFAULT_FERRY_APPS, app + ext)
+            sh.mkdir('-p', os.path.dirname(file_name))
+            with open(file_name, "w") as f:
+                f.write(content)
+            return file_name
+        except IOError as e:
+            logging.error(e)
+            return None
+
     def _read_key_dir(self):
+        """
+        Find the directory containing the container keys. 
+        """
         f = open(ferry.install.DEFAULT_DOCKER_KEY, 'r')
         k = f.read().strip().split("://")
         return k[1], k[0]
@@ -227,6 +293,17 @@ class Installer(object):
         # Check if the user wants to use a specific key directory. 
         self._process_ssh_key(options)
 
+        # Check if the user-application directory exists.
+        # If not, create it. 
+        try:
+            if not os.path.isdir(DEFAULT_FERRY_APPS):
+                os.makedirs(DEFAULT_FERRY_APPS)
+                self._change_permission(DEFAULT_FERRY_APPS)
+        except OSError as e:
+            logging.error("Could not create application directory.\n") 
+            logging.error(e.strerror)
+            sys.exit(1)
+
         # Check if the Mongo directory exists yet. If not
         # go ahead and create it. 
         try:
